function moY = pn( riH, iiN, riS )
%PN sequence generations
%   Y = PN(H) generates the pseudorandom M-sequence with
%       sequence elements 0 and 1 of the maximum length
%
%   Y = PN(H,N) generates the pseudorandom M-sequence with
%       sequence elements 0 and 1 of length N
%
%   Y = PN(H,N,S) same, but with seed S
%
%       H is a vector of primitive generator polynomial;
%         H(1)=h_0, H(2)=h_1, ... , H(m+1)=h_m
%       N is the number of samples to be generated.
%         If N <= 0 or N is not present then the maximum length is generated.
%       S is a seed vector. It does not have to be present.
%
%       Y is the matrix. Each row is the complete state of the
%	  shift register. Number of rows is N. Each column
%         is a PN sequence.

%   Author: Martin Pechanec, October 22, 1997
%           pechanec@iname.com

% -- Check the input parameters
[iR,iC] = size(riH);
if iC == 1,
  riH = riH.';   % Make it a row
end;

[iRH, iCH] = size(riH);
iCH = iCH - 1;
rHgen = riH(1:iCH);

if (nargin == 3),
  [iR, iC] = size(riS);
  if iC == 1,    % Make it row
    riS = riS.';
    iC = iR;
  end

% Set the lower part of riS there
  rReg = zeros( 1, iCH );
  rReg = riS( 1:min( [iCH, iC] ) );
else
  rReg = [ 1, zeros( 1, iCH-1 ) ];
end

% -- Check and set the iiN
if (nargin >= 2),
  if iiN <= 0,
    iiN = 2^iCH - 1;
  end
else
  iiN = 2^iCH - 1;
end

% -- Generate the sequence
moY = zeros( iiN, iCH );
moY(1, :) = rReg;
for iInd = 1:iiN-1,
  iNext = mod( rReg * rHgen', 2 );
  rReg = [ rReg( 2: iCH ), iNext ];
  moY( iInd+1, : ) = rReg;
end

